%{
/*
 *This file is part of the Configuration File Library (CFL).
 * Copyright (C) 2002 Nicolas Darnis <ndarnis@free.fr>
 *
 * The CFL library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * The CFL library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with the CFL library; see the file COPYING.
 * If not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
 *
 * File: scanner.l
 * $Revision: 1.4 $
 * $Date: 2002/11/06 11:45:23 $
 * $Author: ndarnis $
 * $Id: scanner.l,v 1.4 2002/11/06 11:45:23 ndarnis Exp $
 */

#include <config.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <gdsl.h>

#include "cfl.h"
#include "scanner.h"
#include "parser.h"
#include "parser.tab.h"

static int make_string (void);
static int make_identifier (void);
static int make_integer (void);
static int make_real (void);
static int make_boolean (bool val);
%}

%option noyywrap 

space                   [ \t]
sign                    [-+]
letter                  [a-zA-Z_.]
digit                   [0-9]

string                  \"[^\n]*\"
identifier              {letter}({letter}|{digit})*
integer                 {sign}?{digit}+
real                    {integer}?\.{digit}+

%%
{space}+                ;
"#".*                   ;

"["                     return OPEN;
"]"                     return CLOSE;
"="                     return AFFECT;

"FALSE"|"OFF"           return make_boolean (FALSE);
"TRUE"|"ON"             return make_boolean (TRUE);

{string}                return make_string ();
{identifier}            return make_identifier ();
{integer}               return make_integer ();
{real}                  return make_real ();

.                       ;
\n                      ;
<<EOF>>                 yyterminate();
%%

extern void 
yy_set_input_file (const FILE *file)
{
  yy_in = (FILE *) file;
}

static int make_string (void)
{
  yy_lval.string = (char *) malloc (yy_leng - 1);

  if (yy_lval.string != NULL)
  {
    strncpy (yy_lval.string, &yy_text[1], yy_leng - 2);
    yy_lval.string[yy_leng - 2] = '\0';
  }

  return STRING;
}

static int make_identifier (void)
{
  yy_lval.identifier = (char *) malloc (yy_leng + 1);

  if (yy_lval.identifier != NULL)
  {
    strcpy (yy_lval.identifier, yy_text);
  }

  return IDENTIFIER;
}

static int make_integer (void)
{
  long int e = strtol (yy_text, NULL, 0);
  yy_lval.integer = (long int *) malloc (sizeof (long int));

  if (yy_lval.integer != NULL )
  {
    memcpy (yy_lval.integer, &e, sizeof (long int));
  }

  return INTEGER;
}

static int make_real (void)
{
  double f = strtod (yy_text, NULL);
  yy_lval.real = (double *) malloc (sizeof (double));

  if (yy_lval.real != NULL)
  {
    memcpy (yy_lval.real, &f, sizeof (double));
  }

  return REAL;
}

static int make_boolean (bool val)
{
  yy_lval.boolean = (bool *) malloc (sizeof (bool));

  if (yy_lval.boolean != NULL)
  {
    memcpy (yy_lval.boolean, &val, sizeof (bool));
  }

  return BOOLEAN;
}

